@@ -11,7 +11,7 @@ Created on 2020
 # This file contains the main functions concerning the handling of the
 # Gaia mission data.
 #
-# Documentation is provided on Vitral & Macedo, 2021.
+# Documentation is provided on Vitral, 2021.
 # If you have any further questions please email vitral@iap.fr
 #
 ###############################################################################
@@ -45,6 +45,8 @@ def clean_gaia(
     ra,
     dec,
     err_lim=None,
+    err_handle="relative",
+    object_type="gc",
     r_cut=None,
     c=None,
 ):
@@ -79,11 +81,34 @@ def clean_gaia(
         Gaia designation: dec.
     err_lim : float, optional
         Error threshold for proper motions. The default is None.
+    err_handle : string, optional
+        Way in which the error limit is handled:
+            - 'relative', if the err_lim argument is the number of times
+            the initial guess on the galactic object proper motion dispersion.
+            - 'absolute', if the err_lim argument is the absolute error
+            limit in mas/yr.
+        The default is 'relative'.
+    object_type : string, optional
+        Galactic object type, to be considered when defining the default
+        value of err_lim:
+            - 'gc', for globular cluster.
+            - 'dsph', for dwarf spheroidal galaxy.
+        The default is 'gc'.
     r_cut : float, optional
         Projected radius limit. The default is None.
     c : 2D array, optional
         (ra,dec) of the center. The default is None.
 
+    Raises
+    ------
+    ValueError
+        err_handle is not one of the following:
+            - 'relative'
+            - 'absolute'
+        object_type is not one of the following:
+            - 'gc'
+            - 'dsph'
+
     Returns
     -------
     idx : array_like
@@ -92,6 +117,12 @@ def clean_gaia(
 
     """
 
+    if err_handle not in ["relative", "absolute"]:
+        raise ValueError("Does not recognize method to handle errors.")
+
+    if object_type not in ["gc", "dsph"]:
+        raise ValueError("Does not recognize object type.")
+
     if c is None:
         c, unc = position.find_center(ra, dec)
 
@@ -104,7 +135,14 @@ def clean_gaia(
 
     if err_lim is None:
         ini = pm.initial_guess(pmra[idx], pmdec[idx])
-        err_lim = 0.5 * ini[2]
+        if object_type == "gc":
+            err_lim = 0.5 * ini[2]
+        elif object_type == "dsph":
+            err_lim = 5 * ini[2]
+    else:
+        if err_handle == "relative":
+            ini = pm.initial_guess(pmra[idx], pmdec[idx])
+            err_lim = err_lim * ini[2]
 
     u = np.sqrt(chi2 / (nu - 5))
 
@@ -150,12 +188,15 @@ def find_object(
     bw_hrd=None,
     r_max=None,
     err_lim=None,
+    err_handle="relative",
+    object_type="gc",
+    return_center=False,
     check_fit=False,
 ):
 
     """
     Finds galactic object by employing the analysis proposed in
-    Vitral & Macedo 2021.
+    Vitral 2021.
 
     Parameters
     ----------
@@ -188,6 +229,8 @@ def find_object(
              - 'sersic'
              - 'kazantzidis'
              - 'plummer'
+             - 'test', for testing which among Plummer and Sersic should be
+             used, based on AICc.
         The default is 'plummer'.
     min_method : string, optional
         Minimization method to be used by the pm maximum likelihood fit.
@@ -216,6 +259,22 @@ def find_object(
         The default is None
     err_lim : float, optional
         Error threshold for proper motions. The default is None.
+    err_handle : string, optional
+        Way in which the error limit is handled:
+            - 'relative', if the err_lim argument is the number of times
+            the initial guess on the galactic object proper motion dispersion.
+            - 'absolute', if the err_lim argument is the absolute error
+            limit in mas/yr.
+        The default is 'relative'.
+    object_type : string, optional
+        Galactic object type, to be considered when defining the default
+        value of err_lim:
+            - 'gc', for globular cluster.
+            - 'dsph', for dwarf spheroidal galaxy.
+        The default is 'gc'.
+    return_center : boolean, optional
+        True if the user wants an estimate of the center as output.
+        The default is False.
     check_fit : boolean, optional
         True is the user wants to plot validity checks throughout the fitting
         procedure.
@@ -228,12 +287,19 @@ def find_object(
             - 'sersic'
             - 'kazantzidis'
             - 'plummer'
+            - 'test'
         Probability method is not one of the following:
             - 'complete'
             - 'pm'
             - 'position'
         Probability threshold is not inbetween 0 and 1.
         nsig is not positive.
+        err_handle is not one of the following:
+            - 'relative'
+            - 'absolute'
+        object_type is not one of the following:
+            - 'gc'
+            - 'dsph'
 
     Returns
     -------
@@ -251,7 +317,7 @@ def find_object(
 
     """
 
-    if sd_model not in ["sersic", "plummer", "kazantzidis"]:
+    if sd_model not in ["sersic", "plummer", "kazantzidis", "test"]:
         raise ValueError("Does not recognize surface density model.")
 
     if prob_method not in ["complete", "pm", "position"]:
@@ -263,6 +329,12 @@ def find_object(
     if nsig <= 0:
         raise ValueError("nsig must be positive.")
 
+    if err_handle not in ["relative", "absolute"]:
+        raise ValueError("Does not recognize method to handle errors.")
+
+    if object_type not in ["gc", "dsph"]:
+        raise ValueError("Does not recognize object type.")
+
     c, unc = position.find_center(ra, dec)
 
     ri = angle.sky_distance_deg(ra, dec, c[0], c[1])
@@ -310,6 +382,83 @@ def find_object(
             / (np.pi * (10 ** results_sd[0]) ** 2)
         )
         prob_sd = position.prob(ri, results_sd, model="kazantzidis")
+    elif sd_model == "test":
+        results_sd_s, var_sd_s = position.maximum_likelihood(
+            x=np.asarray([ri_sd]), model="sersic"
+        )
+        results_sd_p, var_sd_p = position.maximum_likelihood(
+            x=np.asarray([ri_sd]), model="plummer"
+        )
+        results_sd_k, var_sd_k = position.maximum_likelihood(
+            x=np.asarray([ri_sd]), model="kazantzidis"
+        )
+
+        aicc_s = get_aicc(
+            position.likelihood_sersic(results_sd_s, ri_sd), 3, len(ri_sd)
+        )
+        aicc_p = get_aicc(
+            position.likelihood_plummer(results_sd_p, ri_sd), 2, len(ri_sd)
+        )
+        aicc_k = get_aicc(
+            position.likelihood_kazantzidis(results_sd_k, ri_sd), 2, len(ri_sd)
+        )
+
+        delta_aicc = aicc_p - aicc_s
+
+        if delta_aicc < 2:
+            delta_aicc = aicc_p - aicc_k
+            if delta_aicc < 2:
+                sd_model = "plummer"
+                results_sd, var_sd = results_sd_p, var_sd_p
+                r_cut = 10 * 10 ** results_sd[0]
+                nsys = len(ri_sd) / (1 + 10 ** results_sd[1])
+                nilop = len(ri_sd) - nsys
+                sd = (
+                    position.sd_plummer(sorted(ri_sd) / 10 ** results_sd[0])
+                    * nsys
+                    / (np.pi * (10 ** results_sd[0]) ** 2)
+                )
+                prob_sd = position.prob(ri, results_sd, model="plummer")
+            else:
+                sd_model = "kazantzidis"
+                results_sd, var_sd = results_sd_k, var_sd_k
+                r_cut = 10 * 10 ** results_sd[0]
+                nsys = len(ri_sd) / (1 + 10 ** results_sd[1])
+                nilop = len(ri_sd) - nsys
+                sd = (
+                    position.sd_kazantzidis(sorted(ri_sd) / 10 ** results_sd[0])
+                    * nsys
+                    / (np.pi * (10 ** results_sd[0]) ** 2)
+                )
+                prob_sd = position.prob(ri, results_sd, model="kazantzidis")
+        else:
+            delta_aicc = aicc_k - aicc_s
+            if delta_aicc < 2:
+                sd_model = "kazantzidis"
+                results_sd, var_sd = results_sd_k, var_sd_k
+                r_cut = 10 * 10 ** results_sd[0]
+                nsys = len(ri_sd) / (1 + 10 ** results_sd[1])
+                nilop = len(ri_sd) - nsys
+                sd = (
+                    position.sd_kazantzidis(sorted(ri_sd) / 10 ** results_sd[0])
+                    * nsys
+                    / (np.pi * (10 ** results_sd[0]) ** 2)
+                )
+                prob_sd = position.prob(ri, results_sd, model="kazantzidis")
+            else:
+                sd_model = "sersic"
+                results_sd, var_sd = results_sd_s, var_sd_s
+                r_cut = 10 * 10 ** results_sd[1]
+                nsys = len(ri_sd) / (1 + 10 ** results_sd[2])
+                nilop = len(ri_sd) - nsys
+                sd = (
+                    position.sd_sersic(
+                        results_sd[0], sorted(ri_sd) / 10 ** results_sd[1]
+                    )
+                    * nsys
+                    / (np.pi * (10 ** results_sd[1]) ** 2)
+                )
+                prob_sd = position.prob(ri, results_sd, model="sersic")
 
     if check_fit is True:
         surf_dens = position.surface_density(x=ri_sd)
@@ -361,6 +510,8 @@ def find_object(
         c=c,
         r_cut=r_max,
         err_lim=err_lim,
+        err_handle=err_handle,
+        object_type=object_type,
     )
 
     prob_sd = prob_sd[idx]
@@ -526,7 +677,10 @@ def find_object(
 
         idx_final = idx_p
 
-    return idx_final, results_sd, var_sd, results_pm, var_pm
+    if return_center is False:
+        return idx_final, results_sd, var_sd, results_pm, var_pm
+    else:
+        return idx_final, results_sd, var_sd, results_pm, var_pm, c
 
 
 def extract_object(
@@ -536,10 +690,13 @@ def extract_object(
     prob_method="complete",
     prob_limit=0.9,
     use_hrd=True,
-    nsig=3.3,
+    nsig=3,
     bw_hrd=None,
     r_max=None,
     err_lim=None,
+    err_handle="relative",
+    object_type="gc",
+    return_center=False,
     check_fit=False,
 ):
     """
@@ -554,6 +711,8 @@ def extract_object(
              - 'sersic'
              - 'kazantzidis'
              - 'plummer'
+             - 'test', for testing which among Plummer and Sersic should be
+             used, based on AICc.
         The default is 'plummer'.
     min_method : string, optional
         Minimization method to be used by the pm maximum likelihood fit.
@@ -582,6 +741,22 @@ def extract_object(
         The default is None
     err_lim : float, optional
         Error threshold for proper motions. The default is None.
+    err_handle : string, optional
+        Way in which the error limit is handled:
+            - 'relative', if the err_lim argument is the number of times
+            the initial guess on the galactic object proper motion dispersion.
+            - 'absolute', if the err_lim argument is the absolute error
+            limit in mas/yr.
+        The default is 'relative'.
+    object_type : string, optional
+        Galactic object type, to be considered when defining the default
+        value of err_lim:
+            - 'gc', for globular cluster.
+            - 'dsph', for dwarf spheroidal galaxy.
+        The default is 'gc'.
+    return_center : boolean, optional
+        True if the user wants an estimate of the center as output.
+        The default is False.
     check_fit : boolean, optional
         True is the user wants to plot validity checks throughout the fitting
         procedure.
@@ -594,12 +769,19 @@ def extract_object(
             - 'sersic'
             - 'kazantzidis'
             - 'plummer'
+            - 'test'
         Probability method is not one of the following:
             - 'complete'
             - 'pm'
             - 'position'
         Probability threshold is not inbetween 0 and 1.
         nsig is not positive.
+        err_handle is not one of the following:
+            - 'relative'
+            - 'absolute'
+        object_type is not one of the following:
+            - 'gc'
+            - 'dsph'
 
     Returns
     -------
@@ -617,7 +799,7 @@ def extract_object(
 
     """
 
-    if sd_model not in ["sersic", "plummer", "kazantzidis"]:
+    if sd_model not in ["sersic", "plummer", "kazantzidis", "test"]:
         raise ValueError("Does not recognize surface density model.")
 
     if prob_method not in ["complete", "pm", "position"]:
@@ -629,6 +811,12 @@ def extract_object(
     if nsig <= 0:
         raise ValueError("nsig must be positive.")
 
+    if err_handle not in ["relative", "absolute"]:
+        raise ValueError("Does not recognize method to handle errors.")
+
+    if object_type not in ["gc", "dsph"]:
+        raise ValueError("Does not recognize object type.")
+
     hdu = fits.open(path)
 
     ra = np.asarray(hdu[1].data[:]["ra      "])  # degrees
@@ -648,31 +836,142 @@ def extract_object(
 
     hdu.close()
 
-    idx_final, results_sd, var_sd, results_pm, var_pm = find_object(
-        ra,
-        dec,
-        pmra,
-        pmdec,
-        epmra,
-        epmdec,
-        corrpm,
-        chi2,
-        nu,
-        g_mag,
-        br_mag,
-        br_excess,
-        sd_model=sd_model,
-        min_method=min_method,
-        prob_method=prob_method,
-        prob_limit=prob_limit,
-        use_hrd=use_hrd,
-        nsig=nsig,
-        bw_hrd=bw_hrd,
-        r_max=r_max,
-        err_lim=err_lim,
-        check_fit=check_fit,
-    )
+    if return_center is False:
+        idx_final, results_sd, var_sd, results_pm, var_pm = find_object(
+            ra,
+            dec,
+            pmra,
+            pmdec,
+            epmra,
+            epmdec,
+            corrpm,
+            chi2,
+            nu,
+            g_mag,
+            br_mag,
+            br_excess,
+            sd_model=sd_model,
+            min_method=min_method,
+            prob_method=prob_method,
+            prob_limit=prob_limit,
+            use_hrd=use_hrd,
+            nsig=nsig,
+            bw_hrd=bw_hrd,
+            r_max=r_max,
+            err_lim=err_lim,
+            err_handle=err_handle,
+            object_type=object_type,
+            check_fit=check_fit,
+        )
+    else:
+        idx_final, results_sd, var_sd, results_pm, var_pm, center = find_object(
+            ra,
+            dec,
+            pmra,
+            pmdec,
+            epmra,
+            epmdec,
+            corrpm,
+            chi2,
+            nu,
+            g_mag,
+            br_mag,
+            br_excess,
+            sd_model=sd_model,
+            min_method=min_method,
+            prob_method=prob_method,
+            prob_limit=prob_limit,
+            use_hrd=use_hrd,
+            nsig=nsig,
+            bw_hrd=bw_hrd,
+            r_max=r_max,
+            err_lim=err_lim,
+            err_handle=err_handle,
+            object_type=object_type,
+            return_center=return_center,
+            check_fit=check_fit,
+        )
 
     full_data = full_data[idx_final]
 
-    return full_data, results_sd, var_sd, results_pm, var_pm
+    if return_center is False:
+        return full_data, results_sd, var_sd, results_pm, var_pm
+    else:
+        return full_data, results_sd, var_sd, results_pm, var_pm, center
+
+
+# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+# ---------------------------------------------------------------------------
+"Statistical handling"
+# ---------------------------------------------------------------------------
+
+
+def get_aic(lnL, Nf):
+    """
+    Provides the Akaike Information Criterion (Akaike, 1973), i.e. AIC.
+
+    Parameters
+    ----------
+    lnL : float
+        Minus the log-likelihood.
+    Nf : int
+        Number of free parameters.
+
+    Returns
+    -------
+    AIC : float
+        AIC indicator.
+
+    """
+
+    AIC = 2 * (lnL + Nf)
+    return AIC
+
+
+def get_bic(lnL, Nf, Nd):
+    """
+    Provides the Bayes Information Criterion (Schwarz, 1978), i.e. BIC.
+
+    Parameters
+    ----------
+    lnL : float
+        Minus the log-likelihood.
+    Nf : int
+        Number of free parameters.
+    Nd : int
+        Size of data sample.
+
+    Returns
+    -------
+    BIC : float
+        BIC indicator.
+
+    """
+
+    BIC = 2 * lnL + Nf * np.log(Nd)
+    return BIC
+
+
+def get_aicc(lnL, Nf, Nd):
+    """
+    Provides the corrected Akaike Information Criterion (Sugiyara, 1978),
+    i.e. AICc.
+
+    Parameters
+    ----------
+    lnL : float
+        Minus the log-likelihood.
+    Nf : int
+        Number of free parameters.
+    Nd : int
+        Size of data sample.
+
+    Returns
+    -------
+    AICc : float
+        AICc indicator.
+
+    """
+
+    AICc = get_aic(lnL, Nf) + 2 * Nf * (1 + Nf) / (Nd - Nf - 1)
+    return AICc
